'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _button = require('./button');

var _button2 = _interopRequireDefault(_button);

var _icon = require('./icon');

var _icon2 = _interopRequireDefault(_icon);

var _keys = require('../../util/keys');

var keys = _interopRequireWildcard(_keys);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var MAX_KEY_DELTA = 1000;

var propTypes = _extends({}, _button2.default.propTypes, {
  children: _propTypes2.default.node,
  isRight: _propTypes2.default.bool,
  isTop: _propTypes2.default.bool,
  items: _propTypes2.default.array.isRequired,
  onChange: _propTypes2.default.func.isRequired,
  selectedIndex: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.arrayOf(_propTypes2.default.number)])
});
delete propTypes.onClick;

var Dropdown = (0, _createReactClass2.default)({
  displayName: 'Dropdown',

  propTypes: propTypes,

  getInitialState: function getInitialState() {
    this.filter = '';
    this.lastKeyDown = new Date();

    return {
      isOpen: false
    };
  },
  componentWillUnmount: function componentWillUnmount() {
    window.removeEventListener('keydown', this.onKeyDown);
    document.body.removeEventListener('click', this.onBodyClick);
  },
  onButtonClick: function onButtonClick() {
    if (this.props.items.length > 1) {
      this.setState({ isOpen: true });
      window.addEventListener('keydown', this.onKeyDown);
      document.body.addEventListener('click', this.onBodyClick);
    }
  },
  onKeyDown: function onKeyDown(event) {
    if (event.keyCode === keys.ESC) {
      event.stopPropagation();
      this.close();
    } else {
      var char = String.fromCharCode(event.keyCode);
      if (/[A-Z ]/.test(char)) {
        var now = new Date();
        var delta = now - this.lastKeyDown;
        if (delta < MAX_KEY_DELTA) {
          this.filter += char;
        } else {
          this.filter = char;
        }
        this.lastKeyDown = now;

        var items = this.items.childNodes;
        var itemsLength = items.length;
        for (var i = 0; i < itemsLength; i++) {
          if (items[i].textContent.toUpperCase().indexOf(this.filter) === 0) {
            this.items.scrollTop = items[i].offsetTop;
            break;
          }
        }
      }
    }
  },
  onBodyClick: function onBodyClick(event) {
    if (!Array.isArray(this.props.selectedIndex) || !_reactDom2.default.findDOMNode(this).contains(event.target)) {
      this.close();
    }
  },
  close: function close() {
    this.setState({ isOpen: false });
    window.removeEventListener('keydown', this.onKeyDown);
    document.body.removeEventListener('click', this.onBodyClick);
  },
  onItemClick: function onItemClick(index, item) {
    this.props.onChange(index, item);
  },
  render: function render() {
    var _this = this;

    var dropdownClassName = (0, _classnames2.default)('pl-dropdown', this.props.className, {
      'pl-open': this.state.isOpen,
      'pl-right': this.props.isRight,
      'pl-top': this.props.isTop,
      'pl-optionless': this.props.items.length === 1
    });
    return _react2.default.createElement(
      'div',
      { className: dropdownClassName },
      _react2.default.createElement(
        _button2.default,
        {
          isPrimary: this.props.isPrimary,
          isSecondary: this.props.isSecondary,
          isSmall: this.props.isSmall,
          onClick: this.onButtonClick
        },
        _react2.default.createElement(
          'span',
          { className: 'pl-dropdown-button-wrapper' },
          _react2.default.createElement(
            'span',
            null,
            this.props.children || this.props.items[this.props.selectedIndex]
          ),
          _react2.default.createElement(_icon2.default, { name: 'chevron' })
        )
      ),
      _react2.default.createElement(
        'div',
        { className: 'pl-dropdown-items', ref: function ref(_ref) {
            return _this.items = _ref;
          } },
        this.props.items.map(function (item, index) {
          var _this2 = this;

          var itemClassName = (0, _classnames2.default)('pl-dropdown-item', {
            'pl-selected': Array.isArray(this.props.selectedIndex) ? this.props.selectedIndex.indexOf(index) !== -1 : index === this.props.selectedIndex
          });

          return _react2.default.createElement(
            'div',
            {
              className: itemClassName,
              key: index // eslint-disable-line react/no-array-index-key
              , onClick: function onClick() {
                return _this2.onItemClick(index, item);
              }
            },
            item
          );
        }, this)
      )
    );
  }
});

Dropdown.propTypes = propTypes;

exports.default = Dropdown;